class PanelPicturesController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user, :only => [:new, :edit, :create, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :edit, :create, :update, :destroy]
  else
    before_filter :authenticate_reader, :only => [:index, :show]
    before_filter :authenticate_user, :only => [:new, :edit, :create, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :edit, :create, :update, :destroy]
  end
  before_filter :authenticate_admin!, :only => [:list, :browse]

  def index
    @page = PanelPicture.page params[:page]
    @page_size = PanelPicture.page_size params[:page_size]
    @panel_pictures = PanelPicture.list(@page, @page_size)

    respond_to do |format|
      format.html {
        @paginate = PanelPicture.list_paginate(@page, @page_size)
      }
      format.json { render :json => @panel_pictures.to_json(PanelPicture.list_json_opt) }
    end
  end

  def show
    @panel_picture = PanelPicture.show(params[:id], [@user, @admin])

    respond_to do |format|
      format.html
      format.json { render :json => @panel_picture.to_json(PanelPicture.show_json_opt) }
    end
  end

  def list
    @panel_pictures = PanelPicture.all

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @panel_pictures }
    end
  end

  def browse
    @panel_picture = PanelPicture.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @panel_picture}
    end
  end

  def new
    @picture = Picture.show params[:picture_id], @author
    raise ActiveRecord::Forbidden unless @picture.enable?
    @panel = Panel.edit(@author.working_panel, @author)
    
    @panel_picture = PanelPicture.new :panel_id => @panel.id, :picture_id => @picture.id
    @panel_picture.supply_default

    respond_to do |format|
      format.html
      format.json { render :json => @panel_picture.to_json(PanelPicture.show_json_opt) }
    end
  end

  def edit
    @panel_picture = PanelPicture.show(params[:id], @author)
    @panel = Panel.edit(@panel_picture.panel.id, @author)
    
    respond_to do |format|
      format.html
    end
  end

  def create
    @panel = Panel.edit(@author.working_panel, @author)
    
    @panel_picture = PanelPicture.new 
    @panel_picture.attributes = params[:panel_picture]
    @panel_picture.overwrite @panel.id
    
    @picture = Picture.show @panel_picture.picture_id, @author
    raise ActiveRecord::Forbidden unless @picture.enable?
    
    respond_to do |format|
      if @panel_picture.valid?
        if @panel_picture.store @author
          flash[:notice] = I18n.t('flash.notice.created', :model => Panel.model_name.human)
          format.html { redirect_to @panel }
          format.json { render json: @panel.panel_elements_as_json, status: :created, location: @panel }
        else
          flash[:notice] = I18n.t('flash.notice.not_created', :model => PanelPicture.model_name.human)
          format.html { render action: "new" }
          format.json { render json: @panel.errors, status: :unprocessable_entity }
        end
      else
        flash[:notice] = I18n.t('flash.notice.not_created', :model => PanelPicture.model_name.human)
        format.html { render action: "new" }
        format.json { render json: @panel_picture.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @panel_picture = PanelPicture.show(params[:id], @author)
    @panel_picture.attributes = params[:panel_picture]
    @panel = Panel.edit(@panel_picture.panel.id, @author)
    @panel_picture.overwrite @panel.id
    
    @picture = Picture.show @panel_picture.picture_id, @author
    raise ActiveRecord::Forbidden unless @picture.enable?
    
    respond_to do |format|
      if @panel_picture.store @author
        flash[:notice] = I18n.t('flash.notice.updated', :model => PanelPicture.model_name.human)
        format.html { redirect_to @panel_picture }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_updated', :model => PanelPicture.model_name.human)
        format.html { render action: "edit" }
        format.json { render json: @panel_picture.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @panel_picture = PanelPicture.show(params[:id], @author)
    @panel = Panel.edit(@panel_picture.panel.id, @author)
    
    respond_to do |format|
      if @panel_picture.remove @author
        flash[:notice] = I18n.t('flash.notice.destroyed', :model => PanelPicture.model_name.human)
        format.html { redirect_to @panel }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_destroyed', :model => PanelPicture.model_name.human)
        format.html { redirect_to @panel_picture }
        format.json { render json: @panel_picture.errors, status: :unprocessable_entity }
      end
    end
  end
  
end
