class Scroll < ActiveRecord::Base
  has_many :scroll_panels
  belongs_to :author
  
  validates :title, :presence => true, :length => {:maximum => 100}
  validates :visible, :presence => true, :numericality => true, :inclusion => {:in => 0..1}
  validates :author_id, :presence => true, :numericality => true, :existence => {:both => false}

  before_validation :valid_encode
  
  def valid_encode
    ['title', 'description'].each do |a|
      next if attributes[a] == nil
      raise Pettanr::BadRequest unless attributes[a].valid_encoding?
    end
  end
  
  def supply_default
    self.visible = 0 if self.visible.blank?
  end
  
  def overwrite au
    return false unless au
    self.author_id = au.id
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    au = Scroll.get_author_from_roles roles
    return false unless au
    self.author_id == au.id
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.own?(roles)
    self.visible > 0
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list_where
    'scrolls.visible > 0'
  end
  
  def self.mylist_where au
    ['scrolls.author_id = ?', au.id]
  end
  
  def self.himlist_where au
    ['scrolls.author_id = ? and scrolls.visible > 0', au.id]
  end
  
  def self.list page = 1, page_size = self.default_page_size
    Scroll.where(self.list_where()).includes(Scroll.list_opt).order('scrolls.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.mylist au, page = 1, page_size = Author.default_scroll_page_size
    Scroll.where(self.mylist_where(au)).includes(Scroll.list_opt).order('scrolls.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.himlist au, page = 1, page_size = Author.default_scroll_page_size
    Scroll.where(self.himlist_where(au)).includes(Scroll.list_opt).order('scrolls.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.list_paginate page = 1, page_size = self.default_page_size
    Kaminari.paginate_array(Array.new(Scroll.where(self.list_where()).count, nil)).page(page).per(page_size)
  end
  
  def self.mylist_paginate au, page = 1, page_size = Author.default_scroll_page_size
    Kaminari.paginate_array(Array.new(Scroll.where(self.mylist_where(au)).count, nil)).page(page).per(page_size)
  end
  
  def self.himlist_paginate au, page = 1, page_size = Author.default_scroll_page_size
    Kaminari.paginate_array(Array.new(Scroll.where(self.himlist_where(au)).count, nil)).page(page).per(page_size)
  end
  
  def self.list_opt
    {:scroll_panels => {:panel => {}}, :author => {} }
  end
  
  def self.list_json_opt
    {:include => {:scroll_panels => {:include => {:panel => {}}}, :author => {}}}
  end
  
  def self.show sid, roles
    opt = {}
    opt.merge!(Scroll.show_opt)
    res = Scroll.find(sid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(roles)
    res
  end
  
  def self.edit sid, au
    opt = {}
    opt.merge!(Scroll.show_opt)
    res = Scroll.find(sid, opt)
    raise ActiveRecord::Forbidden unless res.own?(au)
    res
  end
  
  def self.show_opt
    {:include => {:scroll_panels => {:panel => {}}, :author => {}}}
  end
  
  def self.show_json_opt
    {:include => {:scroll_panels => {:include => {:panel => {}}}, :author => {}}}
  end
  
  def self.visible_count
    Scroll.count 'visible > 0'
  end
  
  def destroy_with_scroll_panel
    res = false
    Scroll.transaction do
      self.scroll_panels.each do |scroll_panel|
        raise ActiveRecord::Rollback unless scroll_panel.destroy
      end
      raise ActiveRecord::Rollback unless self.destroy
      res = true
    end
    res
  end
  
  def scenario
    panels.map {|panel|
      panel.scenario
    }.join
  end
  
  def plain_scenario
    panels.map {|panel|
      panel.plain_scenario
    }.join
  end
  
end
