class SheetPanel < ActiveRecord::Base
  belongs_to :author
  belongs_to :panel
  belongs_to :sheet
  
  validates :sheet_id, :numericality => {:allow_blank => true}
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :author_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  def supply_default
    self.sheet_id = nil
    self.panel_id = nil
    self.x = 15
    self.y = 15
    if self.sheet
      self.t = self.sheet.new_t 
      self.z = self.sheet.new_z 
    else
      self.z = 1
      self.t = nil
    end
  end
  
  def overwrite au
    return false unless au
    self.author_id = au.id
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    au = SheetPanel.get_author_from_roles roles
    return false unless au
    self.author_id == au.id
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.sheet.own?(roles)
    self.sheet.visible? roles
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.play_list_where cid
    ['sheet_panels.sheet_id = ?', cid]
  end
  
  def self.list_where
    'sheets.visible > 0'
  end
  
  def self.mylist_where au
    ['sheet_panels.author_id = ?', au.id]
  end
  
  def self.himlist_where au
    ['sheet_panels.author_id = ? and sheets.visible > 0', au.id]
  end
  
  def self.play_list sheet, author
    SheetPanel.where(self.play_list_where(sheet.id)).includes(SheetPanel.list_opt).order('sheet_panels.t')
  end
  
  def self.list page = 1, page_size = self.default_page_size
    SheetPanel.where(self.list_where()).includes(SheetPanel.list_opt).order('sheet_panels.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.mylist au, page = 1, page_size = Author.default_sheet_panel_page_size
    SheetPanel.where(self.mylist_where(au)).includes(SheetPanel.list_opt).order('sheet_panels.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.himlist au, page = 1, page_size = Author.default_sheet_panel_page_size
    SheetPanel.where(self.himlist_where(au)).includes(SheetPanel.list_opt).order('sheet_panels.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.list_paginate page = 1, page_size = self.default_page_size
    Kaminari.paginate_array(Array.new(SheetPanel.where(self.list_where()).includes(SheetPanel.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.mylist_paginate au, page = 1, page_size = Author.default_sheet_panel_page_size
    Kaminari.paginate_array(Array.new(SheetPanel.where(self.mylist_where(au)).includes(SheetPanel.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.himlist_paginate au, page = 1, page_size = Author.default_sheet_panel_page_size
    Kaminari.paginate_array(Array.new(SheetPanel.where(self.himlist_where(au)).includes(SheetPanel.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.list_opt
    {
      :author => {}, 
      :sheet => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }
  end
  
  def self.list_json_opt
    {:include => {
      :author => {}, 
      :sheet => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }}
  end
  
  def self.show spid, roles
    opt = {}
    opt.merge!(SheetPanel.show_opt)
    res = SheetPanel.find spid, opt
    raise ActiveRecord::Forbidden unless res.visible?(roles)
    res
  end
  
  def self.edit spid, au
    opt = {}
    opt.merge!(SheetPanel.show_opt)
    res = SheetPanel.find spid, opt
    raise ActiveRecord::Forbidden unless res.own?(au)
    res
  end
  
  def self.show_opt
    {:include => {
      :author => {}, 
      :sheet => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }}
  end
  
  def elements
    self.panel.elements
  end
  
  def sheet_panel_as_json au
    panel_include = if self.panel and self.panel.visible?(au)
      {:include => {:author => {}}, :methods => :elements}
    else
      {:include => {:author => {}}}
    end
    self.to_json({:include => {:sheet => {:include => {:author => {}}}, :author => {}, :panel => panel_include}})
  end
  
  def self.list_as_json_text ary, au
    '[' + ary.map {|i| i.sheet_panel_as_json(au) }.join(',') + ']'
  end
  
  def licensed_pictures
    if self.panel
      self.panel.licensed_pictures
    else
      {}
    end
  end
  
    def self.panelize elements_attributes
      elements_attributes = [elements_attributes] unless elements_attributes.is_a?(Array)
      hash = {}
      index = 0
      elements_attributes.each do |element_attributes|
        hash[self.to_s.tableize + '_attributes'] ||= {}
        n = if element_attributes['id']
          element_attributes['id'].to_s
        else
          index += 1
          'new' + index.to_s 
        end
        hash[self.to_s.tableize + '_attributes'][n] = element_attributes
      end
      hash
    end
    
    def copy_attributes
      r = self.attributes
      r.delete 'id'
      r.delete 'sheet_id'
      r.delete 'created_at'
      r.delete 'updated_at'
      r
    end
    
  def self.new_t sheet_id
    r = SheetPanel.max_t(sheet_id)
    r.blank? ? 0 : r.to_i + 1
  end
  
  def self.max_t sheet_id
    SheetPanel.maximum(:t, :conditions => ['sheet_id = ?', sheet_id])
  end
  
  def self.find_t sheet_id, t
    SheetPanel.find(:first, :conditions => ['sheet_id = ? and t = ?', sheet_id, t])
  end
  
  def self.collect_t sheet_panel
    r = SheetPanel.find(:all, :conditions => ['sheet_id = ?', sheet_panel.sheet_id], :order => 't')
    r.map {|sp| sp.t}
  end
  
  def self.serial? ary
    i = 0
    ary.compact.sort.each do |t|
      break false unless t == i
      i += 1
    end
    ary.compact.size == i
  end
  
  def self.validate_t sheet_panel
    SheetPanel.serial?(SheetPanel.collect_t(sheet_panel))
  end
  
  def insert_shift
    SheetPanel.update_all('t = t + 1', ['sheet_id = ? and t >= ?', self.sheet_id, self.t])
  end
  
  def lesser_shift old_t
    self.t = 0 if self.t < 0
    SheetPanel.update_all('t = t + 1', ['sheet_id = ? and (t >= ? and t < ?)', self.sheet_id, self.t, old_t])
  end
  
  def higher_shift old_t
    nf = SheetPanel.find_t(self.sheet_id, self.t)
    max_t = SheetPanel.max_t(self.sheet_id).to_i
    self.t = max_t if self.t > max_t
    SheetPanel.update_all('t = t - 1', ['sheet_id = ? and (t > ? and t <= ?)', self.sheet_id, old_t, self.t])
  end
  
  def update_shift old_t
    if self.t > old_t
      higher_shift old_t
    else
      lesser_shift old_t
    end
  end
  
  def rotate old_t = nil
    if self.new_record?
      if self.t.blank?
        self.t = SheetPanel.new_t self.sheet_id
      else
        self.insert_shift
      end
    else
      if self.t.blank?
      else
        self.update_shift old_t
      end
    end
  end
  
  def allow?
    return nil if self.sheet_id == nil or self.panel_id == nil
    self.sheet.own?(self.author) and self.panel.usable?(self.author)
  end
  
  def store old_t = nil
    res = false
    SheetPanel.transaction do
      case self.allow?
      when true
        self.rotate old_t
      when false
        raise ActiveRecord::Forbidden
      else
      end
      res = self.save
      raise ActiveRecord::Rollback unless res
      res = SheetPanel.validate_t(self) 
      unless res
        self.errors.add :t, 'unserialized'
        raise ActiveRecord::Rollback 
      end
    end
    res
  end
  
  def destroy_and_shorten
    res = false
    SheetPanel.transaction do
      SheetPanel.update_all('t = t - 1', ['sheet_id = ? and (t > ?)', self.sheet_id, self.t])
      raise ActiveRecord::Rollback unless self.destroy
      res = true
    end
    res
  end
  
    # element.rb
    def self.colum_structures
      @@colum_structures ||= {
      }
    end
    
    def self.path_name with_engine = false
      self.to_s.tableize
    end
    
    def new_index
      @new_index
    end
    
    def new_index= v
      @new_index = v
    end
    
    def new_sheet
      @new_sheet
    end
    
    def new_sheet= v
      @new_sheet = v
    end
    
    def get_sheet
      self.sheet || @new_sheet
    end
    
    def tag_id c = nil
      'sheet' + self.tag_sheet_id + self.tag_element_type + self.tag_element_id + c.to_s
    end
    
    def field_tag_id f
      self.tag_id + f.to_s
    end
    
    def tag_sheet_id
      self.get_sheet.new_record? ? '0' : self.get_sheet.id.to_s
    end
    
    def tag_element_id
      self.new_record? ? '0' : self.id.to_s
    end
    
    def tag_element_type
      'sheet_panel'
    end
    
    def tag_new_index
      self.new_index.to_s
    end
    
    def path_name with_engine = false
      self.class.path_name(with_engine)
    end
    
    def form_template with_engine = false
      self.path_name(with_engine) + '/form'
    end
    
    def scenario_template with_engine = false
      self.path_name(with_engine) + '/scenario'
    end
    
    def element_face_template with_engine = false
      self.path_name(with_engine) + '/element_face'
    end
    
    def form_helper_template(colum_name)
      self.class.colum_structures[colum_name][:helper]
    end
    
    def tag_attributes column = nil, opt = {}
      {
        :id => self.field_tag_id(column), :sheet_id => self.tag_sheet_id, 
        :element_id => self.tag_element_id, :element_type => self.tag_element_type
      }.merge(opt)
    end
    
    def field_tag_attributes column, no_attr, opt = {}
      self.tag_attributes(column).merge(
        {:column => column, :new_index => self.tag_new_index, :no_attr => no_attr}
      ).merge(opt)
    end
    
    #render element by body
    def any_tag_attributes name = nil, opt = {}
      r = self.tag_attributes(name)
      r.merge!(
        {:new_index => self.tag_new_index}
      ) if self.new_index
      r.merge(opt)
    end
    
    def select_tag_attributes(selected, column, no_attr)
      [
        :last, :first, 
        {:html => {:selected => selected}}, 
        self.field_tag_attributes(column, no_attr)
      ]
    end
    
    def tag_attr column = nil, opt = {}
      self.tag_attributes(column, opt).to_attr
    end
    
    def field_tag_attr column, no_attr, opt = {}
      self.field_tag_attributes(column, no_attr, opt).to_attr
    end
    
    def any_tag_attr name = nil, opt = {}
      self.any_tag_attributes(name, opt).to_attr
    end
    
end
