# -*- encoding: utf-8 -*-
#貸与状況
require 'spec_helper'

describe DemanderStatusesController do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = @user.author    #ユーザ作成時に連動して作成される
    @demand_user = FactoryGirl.create :demand_user
  end
  
  describe '一覧表示に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      DemanderStatus.stub(:list).and_return([@ds, @ds, @ds])
      DemanderStatus.stub(:requested_list).and_return([@ds, @ds])
      sign_in @admin
    end
    context '事前チェックする' do
      it '与えられたpageがセットされている' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq DemanderStatus.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq DemanderStatus.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq DemanderStatus.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it '@hideが空になっている' do
        get :index
        assigns(:hide).should be_blank
      end
      it '貸与状況モデルに全一覧取得を問い合わせている' do
        DemanderStatus.should_receive(:list).exactly(1)
        get :index
      end
      it '@demander_statusesにリストを取得している' do
        get :index
        assigns(:demander_statuses).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '貸与状況モデルにjson一覧出力オプションを問い合わせている' do
          DemanderStatus.should_receive(:list_json_opt).exactly(1)
          get :index, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは貸与状況っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("token").should be_true
          json.first.has_key?("receive_hour1").should be_true
          json.first.has_key?("receive_hour2").should be_true
          json.first.has_key?("requested_at").should be_true
        end
      end
    end
    context '除外フラグが除外のとき' do
      it '@hideが設定されている' do
        get :index, :hide => 1
        assigns(:hide).should_not be_blank
      end
      it '貸与状況モデルに申請中一覧取得を問い合わせている' do
        DemanderStatus.should_receive(:requested_list).exactly(1)
        get :index, :hide => 1
      end
      it '@demander_statusesにリストを取得している' do
        get :index, :hide => 1
        assigns(:demander_statuses).should have_at_least(2).items
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      DemanderStatus.stub(:show).and_return(@ds)
      sign_in @admin
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @ds.id
        response.should be_success
      end
      it '貸与状況モデルに単体取得を問い合わせている' do
        DemanderStatus.should_receive(:show).exactly(1)
        get :show
      end
      it '@demander_statusにアレを取得している' do
        get :show, :id => @ds.id
        assigns(:demander_status).should eq(@ds)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @ds.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @ds.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '貸与状況モデルにjson単体出力オプションを問い合わせている' do
          DemanderStatus.should_receive(:show_json_opt).exactly(1)
          get :show, :id => @ds.id, :format => :json
        end
        it 'データがアレになっている' do
          get :show, :id => @ds.id, :format => :json
          json = JSON.parse response.body
          json.has_key?("token").should be_true
          json.has_key?("receive_hour1").should be_true
          json.has_key?("receive_hour2").should be_true
          json.has_key?("requested_at").should be_true
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @ds.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @ds.id
          response.body.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @ds.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @ds.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '編集フォーム表示に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      sign_in @admin
      DemanderStatus.stub(:edit).with(@ds.id.to_s, @admin).and_return(@ds)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :edit, :id => @ds.id
        response.should be_success 
      end
      it '貸与状況モデルに編集取得を問い合わせている' do
        DemanderStatus.should_receive(:edit).exactly(1)
        get :edit, :id => @ds.id
      end
      it '@demander_statusにデータを用意している' do
        get :edit, :id => @ds.id
        assigns(:demander_status).should eq @ds
      end
      context 'html形式' do
        it 'editテンプレートを描画する' do
          get :edit, :id => @ds.id
          response.should render_template("edit")
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @ds.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :edit, :id => @ds.id
          response.body.should redirect_to '/admins/sign_in'
        end
      end
    end
  end

  describe '更新に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status, :requested_at => Time.now
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      @attr = {:receive_hour1 => 12, :receive_hour2 => 22}
      sign_in @admin
    end
    context '事前チェックしておく' do
      it '貸与状況モデルに編集取得を問い合わせている' do
        DemanderStatus.stub(:edit).with(any_args()).and_return @ds
        DemanderStatus.should_receive(:edit).exactly(1)
        put :update, :id => @ds.id, :demander_status => @attr
      end
      it '貸与状況モデルに貸与承認を依頼している' do
        DemanderStatus.any_instance.stub(:approve).with(any_args).and_return true
        DemanderStatus.any_instance.should_receive(:approve).with('12', '22').exactly(1)
        put :update, :id => @ds.id, :demander_status => @attr
      end
      it '@demander_statusにアレを取得している' do
        put :update, :id => @ds.id, :demander_status => @attr
        assigns(:demander_status).id.should eq(@ds.id)
      end
    end
    context 'つつがなく終わるとき' do
      it '更新される' do
        put :update, :id => @ds.id, :demander_status => @attr
        DemanderStatus.find(@ds.id).receive_hour1.should eq 12
        DemanderStatus.find(@ds.id).receive_hour2.should eq 22
      end
      context 'html形式' do
        before do
        end
        it 'ステータスコード302 Foundを返す' do
          put :update, :id => @ds.id, :demander_status => @attr
          response.status.should eq 302
        end
        it '更新された貸与状況の表示ページへ遷移する' do
          put :update, :id => @ds.id, :demander_status => @attr
          response.should redirect_to(@ds)
        end
        it '一時的メッセージに処理成功を設定する' do
          put :update, :id => @ds.id, :demander_status => @attr
          flash[:notice].should eq I18n.t('demander_statuses.flash.notice.approved')
        end
      end
      context 'json形式' do
        before do
          DemanderStatus.any_instance.stub(:approve).with(any_args()).and_return(true)
        end
        it 'ステータスコード200 OKを返す' do
          put :update, :id => @ds.id, :demander_status => @attr, :format => :json
          response.should be_success 
        end
        it 'ページ本体は特に返さない' do
          put :update, :id => @ds.id, :demander_status => @attr, :format => :json
          response.body.should match /./
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      it 'ステータスコード302 Foundを返す' do
        put :update, :id => @ds.id, :demander_status => @attr
        response.status.should eq 302
      end
      context 'html形式' do
        it 'サインインページへ遷移する' do
          put :update, :id => @ds.id, :demander_status => @attr
          response.body.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
        it '応答メッセージにUnauthorizedを返す' do
          put :update, :id => @ds.id, :demander_status => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗したとき' do
      before do
        DemanderStatus.any_instance.stub(:approve).with(any_args()).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード200 Okを返す' do
          put :update, :id => @ds.id, :demander_status => @attr
          response.status.should eq 200
        end
        it '編集ページを描画する' do
          put :update, :id => @ds.id, :demander_status => @attr
          response.should render_template("edit")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          DemanderStatus.any_instance.stub(:save).and_return(false)
          put :update, :id => @ds.id, :demander_status => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          put :update, :id => @ds.id, :demander_status => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end
  
  describe '貸与停止に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      sign_in @admin
    end
    context '事前チェックしておく' do
      it '貸与状況モデルに編集取得を問い合わせている' do
        DemanderStatus.stub(:edit).with(any_args()).and_return @ds
        DemanderStatus.should_receive(:edit).exactly(1)
        delete :destroy, :id => @ds.id
      end
      it 'モデルに貸与停止を依頼する' do
        DemanderStatus.any_instance.stub(:reject).with(any_args).and_return true
        DemanderStatus.any_instance.should_receive(:reject).exactly(1)
        delete :destroy, :id => @ds.id
      end
      it '@demander_statusにアレを取得している' do
        delete :destroy, :id => @ds.id
        assigns(:demander_status).should eq(@ds)
      end
    end
    context 'つつがなく終わるとき' do
      before do
        DemanderStatus.any_instance.stub(:reject).with(any_args()).and_return(true)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @ds.id
          response.status.should eq 302
        end
        it '貸与状況閲覧(show)ページに遷移する' do
          delete :destroy, :id => @ds.id
          response.should redirect_to(@ds)
        end
        it '一時的メッセージに処理成功を設定する' do
          delete :destroy, :id => @ds.id
          flash[:notice].should eq I18n.t('demander_statuses.flash.notice.rejected')
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          delete :destroy, :id => @ds.id, :format => :json
          response.should be_success 
        end
        it 'ページ本体は特に返さない' do
          delete :destroy, :id => @ds.id, :format => :json
          response.body.should match /./
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      it 'ステータスコード302 Foundを返す' do
        delete :destroy, :id => @ds.id
        response.status.should eq 302
      end
      context 'html形式' do
        it 'サインインページへ遷移する' do
          delete :destroy, :id => @ds.id
          response.body.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
        it '応答メッセージにUnauthorizedを返す' do
          delete :destroy, :id => @ds.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    
    context '否決に失敗した(否決できる状態でなかった)とき' do
      before do
        DemanderStatus.any_instance.stub(:reject).with(any_args).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @ds.id
          response.status.should eq 302
        end
        it '貸与状況閲覧(show)ページに遷移するする' do
          delete :destroy, :id => @ds.id
          response.should redirect_to(@ds)
        end
        it '一時的メッセージに処理失敗を設定する' do
          delete :destroy, :id => @ds.id
          flash[:notice].should eq I18n.t('demander_statuses.flash.notice.not_rejected')
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          delete :destroy, :id => @ds.id, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          delete :destroy, :id => @ds.id, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

end
